import { PlayerInteractionInterface } from './PlayerInteractionInterface';
import { Board } from './entities/Board';
import { PlayerInterface } from './entities/player/PlayerInterface';
import { PlayerMove } from './entities/PlayerMove';
import { Tile } from './entities/Tile';
import { NetworkClientNotifier } from './NetworkClientNotifier';
import { NetworkPlayer } from './entities/player/NetworkPlayer';
import { PlayerMoveSide, PlayerMoveSideType } from './constants';
import { SocketDisconnectedError } from '../common/errors/SocketDisconnectedError';

export class PlayerInteractionNetwork implements PlayerInteractionInterface {
    player: PlayerInterface;
    clientNotifier = new NetworkClientNotifier();

    constructor(player: PlayerInterface) {
        this.player = player;
    }
    
    async makeMove(board: Board): Promise<PlayerMove | null> {
      let response = undefined;
      try {
        response = await this.clientNotifier.sendEventWithAck(this.player as NetworkPlayer, 'ask-client-for-move', {
          freeHands: board.getFreeEnds(),
        });
      } catch (error) {
        throw new SocketDisconnectedError();
      }
      const { tile: tilePlayed, type, direction } = response;
      if (type === 'pass') {
        return null;
      }
      const { player: { hand} } = this;
      const index: number = hand.findIndex(t => t.id === tilePlayed.id); 
      const side: PlayerMoveSideType =  type === 'left' ? PlayerMoveSide.LEFT : PlayerMoveSide.RIGHT;
      const tile = hand.splice(index, 1)[0];
      tile.revealed = true;      
      return board.isValidMove(tile, side, this.player, direction);
    }

    async chooseTile(board: Board): Promise<Tile> {
      const { player: { hand} } = this;
      const response: any = await this.clientNotifier.notifyPlayer(this.player as NetworkPlayer, 'chooseTile');
      const index: number = board.boneyard.findIndex(t => t.id === response.tileId);  
      const tile = board.boneyard.splice(index, 1)[0];
      tile.revealed = true;
      hand.push(tile);
      return tile;
    }
}