import { Request, Response } from "express";
import { BaseController } from "./BaseController";
import { SessionService } from "../services/SessionService";

export class GameController extends BaseController {
  private sessionService: SessionService = new SessionService();

  public async createMatch(req: Request, res: Response) {
    try {
      const { user, body } = req;
      const { sessionName, seed } = body;
      const sessionId = await this.sessionService.createSession(user, sessionName, seed);
      res.status(201).json({ sessionId });
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public joinMatch(req: Request, res: Response) {
    try {
      const { user, body } = req;
      const { sessionId } = body;
      this.sessionService.joinSession(user, sessionId);
      res.status(200).json({ status: 'ok' });
    } catch (error) {
      this.handleError(res, error);
    }
  }

  public listMatches(req: Request, res: Response) {
    try {
      this.sessionService.listSessions().then((sessions) => {
        res.status(200).json(sessions);
      });
    } catch (error) {
      this.handleError(res, error);
    }
  }
}