import { NamespacesService } from "../services/NamespacesService";
import { BaseController } from "./BaseController";
import { Request, Response } from "express";

export class NamespacesController extends BaseController{
  private namespacesService: NamespacesService;
  constructor() {
    super();
    this.namespacesService = new NamespacesService();
  }

  async getNamespaces(req: Request, res: Response) {
    try {
      const namespaces = await this.namespacesService.getNamespaces();
      res.json(namespaces).status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async getNamespace(req: Request, res: Response) {
    try {
      const { id } = req.params;
      const namespace = await this.namespacesService.getNamespace(id);
      res.json(namespace).status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async createNamespace(req: Request, res: Response) {
    try {
      const namespace = req.body;
      const user = req.user;
      await this.namespacesService.createNamespace(namespace, user);
      res.status(201).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async updateNamespace(req: Request, res: Response) {
    try {
      const { body: namespace, params} = req;
      const { id } = params;
      const result = await this.namespacesService.updateNamespace(id, namespace);
      res.status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }

  async deleteNamespace(req: Request, res: Response) {
    try {
      const { id } = req.params;
      await this.namespacesService.deleteNamespace(id);
      res.status(200).end();
    } catch (error) {
      this.handleError(res, error);
    }
  }
}