import { ObjectId } from "mongodb";

export interface Entity {
  createdAt?: number | null;
  modifiedAt?: number | null;
  createdBy?: ObjectId | null;
  modifiedBy?: ObjectId | null;
}

export interface EntityMongo extends Entity {
  _id?: ObjectId;
}

export interface Score {
  player: string;
  score: number;
}

export interface Namespace extends EntityMongo {
  name?: string;
  description?: string;
  default: boolean;
  type: string | null;
  ownerId?: ObjectId;
  users?: any[];
}

export interface User extends EntityMongo {
  id: string,
  username: string;
  namespaceId: ObjectId;
  hash?: string;
  roles: string[];
  firstname?: string;
  lastname?: string;
  email?: string;
  profileId?: string;
  password?: string | null;
  namespace?: Namespace;
}

export interface DbMatchSession extends EntityMongo {
  id: string;
  name: string;
  creator: string;
  players: string[];
  seed: string;
  mode: string;
  pointsToWin: number;
  maxPlayers: number;
  numPlayers: number;
  scoreboard: Score[];
  matchWinner: string | null;
  state: string;
}

export interface DbUser extends EntityMongo {
  id: string,
  username: string;
  namespaceId: ObjectId;
  hash?: string;
  roles: string[];
  firstname?: string;
  lastname?: string;
  email?: string;
  profileId?: string;
  password?: string | null;
  namespace?: DbNamespace;
}

export interface DbNamespace extends EntityMongo {
  name?: string;
  description?: string;
  default: boolean;
  type: string | null;
  ownerId?: ObjectId;
}

export interface Token extends EntityMongo {
  token: string;
  userId?: ObjectId;
  roles?: string[];
  expiresAt?: number | null;
  type: string;
  namespaceId?: ObjectId
  description?: string;
}

export interface AuthenticationOption {
  roles?: string[];
}

export interface Role {
  name: string;
  description: string;
  permissions: string[];
}

