import { mongoExecute } from './common/mongoDBPool';
import { PipelineLibrary } from './common/PipelineLibrary';
import { BaseMongoManager } from './common/BaseMongoManager';
import { User } from '../interfaces';

export class UsersMongoManager extends BaseMongoManager {
  collection = 'users';

  addUser(user: User) {
    return this.create(user);
  }

  updateUser(user: User) {
    return this.update(user);
  }  

  async updateUserNamespace(userId: string, namespaceId: string) {
    return await mongoExecute(async ({ collection }) => {
      return await
        collection?.updateOne({ _id: this.toObjectId(userId) }, { $set: { namespaceId: this.toObjectId(namespaceId) } });
    }
    , { colName: this.collection });
  }
  
  async deleteUser(_id: string) {
    return await mongoExecute(async ({ collection }) => {
      await collection?.deleteOne({ _id: this.toObjectId(_id) });
    }, { colName: this.collection });
  }

  async getAvailableUsers(defaultId: string): Promise<User[]>{
    return await mongoExecute(async ({ collection }) => {
      const pipeline =  PipelineLibrary.usersGetAvailableUsers(this.toObjectId(defaultId));
      return await collection?.aggregate(pipeline).toArray();
    }, { colName: this.collection });
  }

  async getUsers(): Promise<User[]> {
    return await mongoExecute(async ({ collection }) => {
      const pipeline =  PipelineLibrary.usersGetUsers();
      return await collection?.aggregate(pipeline).toArray();
    }, { colName: this.collection });
  }

  async getById(id: string): Promise<User | null>{
    const pipeline = PipelineLibrary.usersGetById(this.toObjectId(id));
    return await mongoExecute(async ({ collection }) => {
      const users = await collection?.aggregate(pipeline).toArray();
      if (users === undefined || users.length === 0) {
        return null;
      }
      const user = users[0];
      delete user.hash;
      return user;
    }
    , { colName: this.collection });
  }

  async getByUsername(username: string): Promise<User | null>{
    return await mongoExecute(async ({ collection }) => {
      return await collection?.findOne({ username });
    }, { colName: this.collection });
  }

  async updatePassword(username: string, hash: string) {
    return await mongoExecute(async ({ collection }) => {
      return await collection?.updateOne({ username }, { $set: { hash } });
    }, { colName: this.collection });
  } 
}