import { ObjectId } from "mongodb";
import { mongoExecute } from "./mongoDBPool";
import { Entity } from "../../interfaces";
import { LoggingService } from "../../../../common/LoggingService";
import toObjectId from "./mongoUtils";


export abstract class BaseMongoManager {
  protected abstract collection?: string;
  logger = new LoggingService().logger;

  create(data: Entity): Promise<ObjectId | undefined>{
    return mongoExecute(
      async ({ collection }) => {
        const result = await collection?.insertOne(data as any);
        return result?.insertedId;
      },
      { colName: this.collection }
    );
  }

  delete(id: string) {
    return mongoExecute(
      async ({ collection }) => {
        await collection?.deleteOne({ _id: this.toObjectId(id) });
      },
      { colName: this.collection }
    );
  }

  deleteByFilter(filter: any) {
    return mongoExecute(
      async ({ collection }) => {
        await collection?.deleteOne(filter);
      },
      { colName: this.collection }
    );
  }

  getById(id: string) {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.findOne({ _id: this.toObjectId(id) });
      },
      { colName: this.collection }
    );
  }

  getByFilter(filter: any) {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.findOne(filter);
      },
      { colName: this.collection }
    );
  }

  list() {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.find().toArray();
      },
      { colName: this.collection }
    );
  }

  listByFilter(filter: any) {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.find(filter).toArray();
      },
      { colName: this.collection }
    );
  }

  update(object: Entity) {
    const data: any  = { ...object };
    const id = data._id;
    delete data._id;    
    return mongoExecute(async ({ collection }) => {
      return await collection?.updateOne(
        { _id: this.toObjectId(id) },
        { $set: data }
      );
    },
    { colName: this.collection });
  }

  updateMany(filter: any, data: Entity) {
    return mongoExecute(async ({ collection }) => {
      return await collection?.updateMany(filter, { $set: data as any });
    },
    { colName: this.collection });
  }
  
  replaceOne(filter: any, object: Entity) {
    return mongoExecute(async ({collection}) => {
      return await collection?.replaceOne(filter, object);
    }, {colName: this.collection});
  }

  aggregation(pipeline: any) {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.aggregate(pipeline).toArray();
      },
      { colName: this.collection }
    );
  }

  aggregationOne(pipeline: any) {
    return mongoExecute(
      async ({ collection }) => {
        return await collection?.aggregate(pipeline).next();
      },
      { colName: this.collection }
    );
  }

  protected toObjectId = (oid: string) => {
    return toObjectId(oid);    
  };
}
