import crypto from 'crypto';
import jwt from 'jsonwebtoken';
import bcrypt from 'bcryptjs';
import { User } from '../db/interfaces';

export class SecurityManager {
  saltRounds = Number(process.env.SALT_ROUNDS);
  jwtSecretKey = process.env.JWT_SECRET_KEY || '';

  generateId() {
    return crypto.randomBytes(16).toString('hex');
  }  

  getHashedPassword(password: string) {
    const salt = bcrypt.genSaltSync(this.saltRounds);
    return bcrypt.hashSync(password, salt);
  }

  generateApiToken() {
    return crypto.randomBytes(32).toString('hex');
  }

  signJwt(data: any) {
    return jwt.sign(data, this.jwtSecretKey, { expiresIn: '3h' });
  }

  // TODO: verificar esto
  async verifyJwt(token: string): Promise<User> {
    return new Promise((resolve, reject) => {
      jwt.verify(token, this.jwtSecretKey, (err, decoded) => {
        if (err) {
          reject(err);
        } else {
          resolve(decoded as User);
        }
      });
    });
  }
}