import { Namespace, User } from "../db/interfaces";
import { NamespacesMongoManager } from "../db/mongo/NamespacesMongoManager";
import { UsersService } from "./UsersService";

export class NamespacesService {
  namespacesManager = new NamespacesMongoManager();
  usersService = new UsersService();

  async createNamespace(namespace: Namespace, user: User) {
    const insertedId = await this.namespacesManager.createNamespace({ ownerId: user._id, ...namespace, createdBy: user._id });
    await this._updateNamespaceUsers(namespace.users ?? [], insertedId);
    return insertedId;
  }

  async _updateNamespaceUsers(users: any[], id: string) {
    const defaultNamespace: Namespace = await this.namespacesManager.getDefaultNamespace();
    for (const user of users) {
      if (defaultNamespace._id === undefined) continue;
      const namespaceId = user.removed ? defaultNamespace._id?.toString() : id;
      await this.usersService.updateUserNamespace(user.id, namespaceId);
    }    
  }

  async updateNamespace(id: string, namespace: Namespace) {
    const result = await this.namespacesManager.updateNamespace(id, namespace);
    await this._updateNamespaceUsers(namespace.users ?? [], id);
    return result;
  }

  async getNamespace(namespaceId: string) {
    return await this.namespacesManager.getNamespace(namespaceId);
  }

  async getNamespaces() {
    return await this.namespacesManager.getNamespaces();
  }

  getDefaultNamespace(): Promise<Namespace> {
    return this.namespacesManager.getDefaultNamespace();
  }

  async deleteNamespace(namespaceId: string) {
    return await this.namespacesManager.deleteNamespace(namespaceId);
  }
}