import { DominoesGame } from "../../game/DominoesGame";
import { MatchSession } from "../../game/MatchSession";
import { GameState } from "../../game/dto/GameState";
import { PlayerInterface } from "../../game/entities/player/PlayerInterface";

export class PlayerNotificationService {

  async notifyGameState(game: DominoesGame) {
    const gameState: GameState = game.getGameState();
    const { players } = game;
    let promises: Promise<void>[] = players.map(player => player.sendEventWithAck('update-game-state', gameState));
    return await Promise.all(promises);
  }

  async notifyPlayersState(players: PlayerInterface[]) {
    let promises: Promise<void>[] = players.map(player => player.sendEventWithAck('update-player-state', player.getState()));
    return await Promise.all(promises);
  }


  async notifyMatchState(session: MatchSession) {
    const { players } = session;
    let promises: Promise<void>[] = players.map(player => player.sendEventWithAck('update-match-session-state', session.getState()));
    return await Promise.all(promises);
  }

  async sendEventToPlayers(event: string, players: PlayerInterface[], data: any = {}) {
    let promises: Promise<void>[] = players.map(player => player.sendEvent(event, data));
    return await Promise.all(promises);
  }

  async sendEvent(event: string, player: PlayerInterface, data: any = {}) {  
    player.sendEvent(event, data)
  }
}